use crate::listener::*;
use crate::*;
use std::collections::{HashMap, HashSet};
use std::env;
use std::fmt;
use std::io::prelude::*;
use std::io::{BufRead, BufReader, Error, ErrorKind, Result};
use std::net::SocketAddr;
use std::os::unix::net::UnixStream;
use std::path::PathBuf;
use std::{thread, time};

pub(crate) fn gen_events(
    state: &HashMap<String, Peer>,
    prev: &HashMap<String, Peer>,
    listeners: &Vec<Box<EventListener>>,
    timeout: time::Duration,
) {
    let side_by_side = {
        state
            .keys()
            .map(String::as_ref)
            .chain(prev.keys().map(String::as_ref))
            .collect::<HashSet<&str>>()
            .iter()
            .map(|p| (p.to_owned(), (prev.get(*p), state.get(*p))))
            .collect::<HashMap<&str, (Option<&Peer>, Option<&Peer>)>>()
    };
    for (_id, (prev, cur)) in side_by_side {
        /*if id != "HhRgEL2xsnEIqThSTUKLGaTXusorM1MFdjSSYvzBynY=" {
            continue;
        }
        println!("{} p {} c {}", _id, prev.is_some(), cur.is_some());*/
        match (prev, cur) {
            (Some(prev), Some(cur)) if prev.endpoint != cur.endpoint => {
                if let (Some(prev_addr), Some(_)) = (prev.endpoint, cur.endpoint) {
                    listeners.roaming(&cur, prev_addr);
                }
            }
            (Some(prev), Some(cur)) => {
                //shake > timeout && prev.shake < timeout -> listeners.iter().for_each(|l| l.disconnected(&cur));
                //shake < timeout && (prev.shake is none || prev.shake > timeout) -> listeners.iter().for_each(|l| l.connected(&cur));
                if let (Some(shake), Some(pshake)) = (cur.last_handshake, prev.last_handshake) {
                    if shake > timeout && pshake < timeout {
                        listeners.disconnected(&cur);
                    }
                    continue;
                }
                if let Some(shake) = cur.last_handshake {
                    if shake > timeout
                        && prev
                            .last_handshake
                            .map(|shake| shake > timeout)
                            .unwrap_or(true)
                    {
                        listeners.connected(&cur);
                    }
                    continue;
                }
            }
            #[cfg(addrem)]
            (None, Some(cur)) => listeners.added(&cur),
            #[cfg(addrem)]
            (Some(prev), None) => listeners.removed(&prev),
            #[cfg(not(addrem))]
            (None, Some(_cur)) => (),
            #[cfg(not(addrem))]
            (Some(_prev), None) => (),
            fail => {
                println!("{:?}", fail);
                unreachable!()
            }
        }
    }
}
